/********************************************************************
 *                   															                  *
 *                      (c)2005 Maarten Sierhuis										*
 *                        All Rights Reserved                       *
 *                                                                  *
 *  This program cexplains the use of Composite Activities and      *
 *  interruption and resumption of workframes, based on priorities  *
 *  and the workframe-activity context switching of the Brahms VM.  *
 *  This program was written for the SIMS, UC Berkeley course       *
 *  Agent-based Modeling and Simulation of Organizations and Work		*
 *  Work Practice                                                   *
 *                                                                  *
 ********************************************************************/

package PrimitiveActivityPerformer;

/**
 * This group provides a set of primitive activities that are performed
 * to show the effect of interruption of workframes and primitive activities.
 *
 * @author Maarten Sierhuis
 * @version 15 March, 2005
 */
group PrimitiveActivityPerformer memberof SystemGroup {

	attributes:
		public boolean execute_PAC_1; //attribute to state whether to execute PAC_1 or not
		public int nr_pac_2; //PAC_2 counter
		public int nr_pac_1; //PAC_1 counter
		public double pac_1_2_ratio; //attribute to calculate how many times PAC_1 has been performed (# PAC_1 / # PAC_2)
		
	initial_beliefs:
		(current.nr_pac_1 = 0); //intialize the counter
		(current.nr_pac_2 = 0); //initialize the counter

	activities:
	
		/**
		 * Primitive Activity PAC_1. 
		 * This activitiy shows the use of the display and priority facets.
		 * It also shows how to pass the priority as a paramter.
		 *
		 * @param pri the priority of the activity
		 * 
		 */	
		primitive_activity PAC_1(int pri) {
			display: "PAC 1";
			priority: pri;
			max_duration: 900;
		}
		
		/**
		 * Primitive Activity PAC_2. 
		 * This activitiy shows the use of the display, priority and max_duration facets.
		 * It also shows how to pass the priority and duration as a paramter.
		 *
		 * @param pri the priority of the activity
		 * @param dur the duration of the activity
		 * 
		 */	
		primitive_activity PAC_2(int pri, int dur) {
			display: "PAC 2";
			priority: pri;
			max_duration: dur;
		}

	workframes:
	
		/**
		 * Workframe wf_PAC_1. 
		 * This workframe executes the PAC_1 activity with priority 1.
		 * This workframe fires when the agent gets the belief (current.execute_PAC_1 = true)
		 * The other preconditions are only there to calculate values that can then be used
		 * in the body of the workframe. This is done, because the Brahms language can currently not
		 * perform complext arithmetic functions in the body of a workframe. Specifically, assignments
		 * of variables need to be done in the preconditions.
		 * 
		 */	
		workframe wf_PAC_1 {
			repeat: true;
			
			variables:
				forone(int) i; // # of PAC_1 executions
				forone(int) y; // # of PAC_2 executions
				forone(int) i_plus_one; // (i+1)
					
			when (knownval(current.execute_PAC_1 = true) and
						knownval(current.nr_pac_1 = i) and
						knownval(current.nr_pac_2 = y) and
						knownval(i_plus_one = i + 1))
			do {
				PAC_1(1);
				conclude((current.nr_pac_1 = i + 1)); //add 1 to the # of PAC_1 executions
				printBelief(current, nr_pac_1, attribute); //print the belief (current.nr_pac_1 = [value]), look in SystemGroup for the activity
				conclude((current.pac_1_2_ratio = i_plus_one / y)); //calculate the ratio of PAC_1 to PAC_2 executions
				printBelief(current, pac_1_2_ratio, attribute); //print the belief pac_1_2_ratio = [value]), look in SystemGroup for the activity
				conclude((current.execute_PAC_1 = false)); //make sure the workframe does not end in a end-less loop
			} //end do
		} //end wf_PAC_1

		/**
		 * Workframe wf_PAC_1. 
		 * This workframe executes the PAC_2 activity twice.
		 * The idea is that this workframe gets interrupted by wf_PAC_1 ones in
		 * the four times (i.e 25%). This is done by generating the (current.execute_PAC_1 = true)
		 * with a bc=25. When the belief gets generated, the workframe is interrupted, because
		 * wf_PAC_1 has a higher priority activity in it (activity PAC_1 has priority 1, while
		 * PAC_2 has priority 0.
		 * After wf_PAC_1 is finished executing, this workframe continues and the second PAC_2 activity
		 * gets executed.
		 */				
		workframe wf_PAC_2 {
			repeat: true;
			
			do {
				PAC_2(0, 1800);
				conclude((current.nr_pac_2 = current.nr_pac_2 + 1)); //add 1 to the # of PAC_2 executions
				printBelief(current, nr_pac_2, attribute); //print the belief (current.nr_pac_2 = [value]), look in SystemGroup for the activity
				conclude((current.execute_PAC_1 = true), bc:25); //generate the belief to start execution of wf_PAC_1 25% of the time
				PAC_2(0, 600);
			} //end do
		} //end wf_PAC_2
}
